<?php

class IdehPayamApiClient
{
    public static $_TYPE_SOAP = 1;
    public static $_TYPE_REST = 2;

    private static $_restUrl = 'http://localhost:8800/idehpayam/new-api/rest.php';
    private static $_soapUrl = 'http://localhost:8800/idehpayam/new-api/soap.php?wsdl';

    private $_type = 1;
    private $_username = '';
    private $_password = '';
    private $_client = null;
    private $_lastError = null;
    private $_lastResponse = null;

    public function __construct(string $username, string $password, int $type = 1)
    {
        if ($type !== self::$_TYPE_SOAP && $type !== self::$_TYPE_REST) {
            throw new Exception('type parameter out of bound');
        }

        $this->_username = $username;
        $this->_password = $password;
        $this->_type = $type;

        switch ($type) {
            case self::$_TYPE_SOAP:
                $this->_client = new SoapClient(self::$_soapUrl, [
                    'encoding'              => 'UTF-8',
                    'connection_timeout'    => 10,
                    'use'                   => SOAP_LITERAL,
                    'style'                 => SOAP_DOCUMENT,
                    'trace'                 => 1,
                    'exceptions'            => 1
                ]);
            break;
            case self::$_TYPE_REST:
            break;
        }
    }

    public function GetLastError(): ?string
    {
        return $this->_lastError;
    }

    public function GetLastResponse(): ?string
    {
        return $this->_lastResponse;
    }

    /**
     * Get user credit or null if fails
     * 
     * @return float|null
     */
    public function GetUserCredit(): ?float
    {
        $credit = null;

        switch ($this->_type) {
            case self::$_TYPE_SOAP:
                try {
                    $response = $this->_client->__call('GetUserCredit', [
                        "username" => $this->_username,
                        "password" => $this->_password
                    ]);
        
                    if (isset($response->status)) {
                        switch ($response->status) {
                            case 0:
                                return $response->value;
                            break;
                        }
                    } else {
                        $this->_lastError = 'Server error: status does not set.';
                    }
                } catch (Exception $exp) {
                    $this->_lastError = 'Server error: ' . var_export($exp, true);
                }

                $this->_lastResponse = $this->_client->__getLastResponse();
            break;
            case self::$_TYPE_REST:
                try {
                    $response = file_get_contents(self::$_restUrl, false, stream_context_create([
                        'http' => [
                            'method' => 'POST',
                            'header'  => "Content-type: application/json",
                            'content' => json_encode([
                                'operation' => 'GetUserCredit',
                                'username' => $this->_username,
                                'password' => $this->_password,
                            ])
                        ]
                    ]));

                    $this->_lastResponse = $response;

                    if ($responseObject = json_decode($response)) {
                        if (isset($responseObject->status) && isset($responseObject->result->status)) {
                            $credit = $responseObject->result->value;
                        } else {
                            $this->_lastError = 'Server error: status does not set.';
                        }
                    } else {
                        $this->_lastError = 'Server error: response is not json.';
                    }
                } catch (Exception $exp) {
                    $this->_lastError = 'Server error: ' . var_export($exp, true);
                }
            break;
        }

        return $credit;
    }
}

$client = new IdehPayamApiClient('prodo', '123456', IdehPayamApiClient::$_TYPE_REST);
$credit = $client->GetUserCredit();
if ($credit === null) {
    echo $client->getLastError();
} else {
    echo $credit;
}